@@ -362,17 +362,13 @@ gst_init_get_option_group (void)
 
   /* Since GLib 2.23.2 calling g_thread_init() 'late' is allowed and is
    * automatically done as part of g_type_init() */
-  if (!glib_check_version (2, 23, 3)) {
+  if (glib_check_version (2, 23, 3)) {
     /* The GLib threading system must be initialised before calling any other
      * GLib function according to the documentation; if the application hasn't
      * called gst_init() yet or initialised the threading system otherwise, we
      * better issue a warning here (since chances are high that the application
      * has already called other GLib functions such as g_option_context_new() */
-#if GLIB_CHECK_VERSION (2,20,0)
     if (!g_thread_get_initialized ()) {
-#else
-    if (!g_thread_supported ()) {
-#endif
       g_warning ("The GStreamer function gst_init_get_option_group() was\n"
           "\tcalled, but the GLib threading system has not been initialised\n"
           "\tyet, something that must happen before any other GLib function\n"
@@ -428,11 +424,7 @@ gst_init_check (int *argc, char **argv[], GError ** err)
 #endif
   gboolean res;
 
-#if GLIB_CHECK_VERSION (2,20,0)
   if (!g_thread_get_initialized ())
-#else
-  if (!g_thread_supported ())
-#endif
     g_thread_init (NULL);
 
   if (gst_initialized) {
@@ -508,6 +500,22 @@ gst_init (int *argc, char **argv[])
   }
 }
 
+/**
+ * gst_is_initialized:
+ *
+ * Use this function to check if GStreamer has been initialized with gst_init()
+ * or gst_init_check().
+ *
+ * Returns: TRUE if initialization has been done, FALSE otherwise.
+ *
+ * Since: 0.10.31
+ */
+gboolean
+gst_is_initialized (void)
+{
+  return gst_initialized;
+}
+
 #ifndef GST_DISABLE_REGISTRY
 static void
 add_path_func (gpointer data, gpointer user_data)
@@ -577,11 +585,8 @@ init_pre (GOptionContext * context, GOptionGroup * group, gpointer data,
   g_type_init ();
 
   /* we need threading to be enabled right here */
-#if GLIB_CHECK_VERSION (2,20,0)
   g_assert (g_thread_get_initialized ());
-#else
-  g_assert (g_thread_supported ());
-#endif
+
   _gst_debug_init ();
 
 #ifdef ENABLE_NLS
@@ -724,6 +729,7 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   g_type_class_ref (gst_message_type_get_type ());
   g_type_class_ref (gst_mini_object_flags_get_type ());
   g_type_class_ref (gst_pad_link_return_get_type ());
+  g_type_class_ref (gst_pad_link_check_get_type ());
   g_type_class_ref (gst_flow_return_get_type ());
   g_type_class_ref (gst_activate_mode_get_type ());
   g_type_class_ref (gst_pad_direction_get_type ());
@@ -792,9 +798,9 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   }
 #endif /* GST_DISABLE_TRACE */
 
-  GST_INFO ("GLib runtime version: %d.%d.%d\n", glib_major_version,
+  GST_INFO ("GLib runtime version: %d.%d.%d", glib_major_version,
       glib_minor_version, glib_micro_version);
-  GST_INFO ("GLib headers version: %d.%d.%d\n", GLIB_MAJOR_VERSION,
+  GST_INFO ("GLib headers version: %d.%d.%d", GLIB_MAJOR_VERSION,
       GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
 
   return TRUE;
@@ -1090,6 +1096,7 @@ gst_deinit (void)
   g_type_class_unref (g_type_class_peek (gst_message_type_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_mini_object_flags_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_pad_link_return_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pad_link_check_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_flow_return_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_activate_mode_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_pad_direction_get_type ()));
