@@ -41,7 +41,7 @@ G_BEGIN_DECLS
 
 /* make sure we don't change the object size but still make it compile
  * without libxml */
-#ifdef GST_DISABLE_LOADSAVE
+#if defined(GST_DISABLE_LOADSAVE) || defined(GST_DISABLE_DEPRECATED)
 #define GstXmlNodePtr	gpointer
 #else
 #define GstXmlNodePtr	xmlNodePtr
@@ -99,7 +99,7 @@ typedef enum
 #define GST_OBJECT_LOCK(obj)                   g_mutex_lock(GST_OBJECT_GET_LOCK(obj))
 /**
  * GST_OBJECT_TRYLOCK:
- * @obj: a #Object.
+ * @obj: a #GstObject.
  *
  * This macro will try to obtain a lock on the object, but will return with
  * FALSE if it can't get it immediately.
@@ -186,7 +186,7 @@ typedef struct _GstObjectClass GstObjectClass;
  * @refcount: unused
  * @lock: object LOCK
  * @name: The name of the object
- * @name_prefix: used for debugging
+ * @name_prefix: unused
  * @parent: this object's parent, weak ref
  * @flags: use GST_OBJECT_IS_XXX macros to access the flags
  *
@@ -196,12 +196,12 @@ struct _GstObject {
   GObject 	 object;
 
   /*< public >*/
-  gint           refcount;
+  gint           refcount;    /* unused (FIXME 0.11: remove) */
 
   /*< public >*/ /* with LOCK */
   GMutex        *lock;        /* object LOCK */
   gchar         *name;        /* object name */
-  gchar         *name_prefix; /* used for debugging */
+  gchar         *name_prefix; /* (un)used for debugging (FIXME 0.11: remove) */
   GstObject     *parent;      /* this object's parent, weak ref */
   guint32        flags;
 
@@ -239,29 +239,41 @@ struct _GstObject {
  */
 #define GST_CLASS_UNLOCK(obj)           (g_static_rec_mutex_unlock(GST_CLASS_GET_LOCK(obj)))
 
-/*
+/**
  * GstObjectClass:
- *
+ * @parent_class: parent
+ * @path_string_separator: separator used by gst_object_get_path_string()
  * @signal_object: is used to signal to the whole class
+ * @lock: class lock to be used with GST_CLASS_GET_LOCK(), GST_CLASS_LOCK(), GST_CLASS_UNLOCK() and others.
+ * @parent_set: default signal handler
+ * @parent_unset: default signal handler
+ * @object_saved: default signal handler
+ * @deep_notify: default signal handler
  * @save_thyself: xml serialisation
  * @restore_thyself: xml de-serialisation
+ *
+ * GStreamer base object class.
  */
 struct _GstObjectClass {
   GObjectClass	parent_class;
 
-  gchar		*path_string_separator;
+  const gchar	*path_string_separator;
   GObject	*signal_object;
 
+  /* FIXME-0.11: remove this, plus the above GST_CLASS_*_LOCK macros */
   GStaticRecMutex *lock;
 
   /* signals */
+  /* FIXME-0.11: remove, and pass NULL in g_signal_new(), we never used them */
   void          (*parent_set)       (GstObject * object, GstObject * parent);
   void          (*parent_unset)     (GstObject * object, GstObject * parent);
+  /* FIXME 0.11: Remove this, it's deprecated */
   void          (*object_saved)     (GstObject * object, GstXmlNodePtr parent);
   void          (*deep_notify)      (GstObject * object, GstObject * orig, GParamSpec * pspec);
 
   /*< public >*/
   /* virtual methods for subclasses */
+  /* FIXME 0.11: Remove this, it's deprecated */
   GstXmlNodePtr (*save_thyself)     (GstObject * object, GstXmlNodePtr parent);
   void          (*restore_thyself)  (GstObject * object, GstXmlNodePtr self);
 
@@ -275,8 +287,11 @@ GType		gst_object_get_type		(void);
 /* name routines */
 gboolean	gst_object_set_name		(GstObject *object, const gchar *name);
 gchar*		gst_object_get_name		(GstObject *object);
+
+#ifndef GST_DISABLE_DEPRECATED
 void		gst_object_set_name_prefix	(GstObject *object, const gchar *name_prefix);
 gchar*		gst_object_get_name_prefix	(GstObject *object);
+#endif
 
 /* parentage routines */
 gboolean	gst_object_set_parent		(GstObject *object, GstObject *parent);
@@ -303,6 +318,7 @@ gchar *		gst_object_get_path_string	(GstObject *object);
 gboolean	gst_object_check_uniqueness	(GList *list, const gchar *name);
 
 /* load/save */
+#ifndef GST_DISABLE_DEPRECATED
 #ifndef GST_DISABLE_LOADSAVE
 GstXmlNodePtr   gst_object_save_thyself    (GstObject *object, GstXmlNodePtr parent);
 void            gst_object_restore_thyself (GstObject *object, GstXmlNodePtr self);
@@ -312,6 +328,7 @@ void            gst_object_restore_thyself (GstObject *object, GstXmlNodePtr sel
 #pragma GCC poison gst_object_restore_thyself
 #endif
 #endif
+#endif
 
 /* class signal stuff */
 guint		gst_class_signal_connect	(GstObjectClass	*klass,
@@ -319,6 +336,7 @@ guint		gst_class_signal_connect	(GstObjectClass	*klass,
 						 gpointer	 func,
 						 gpointer	 func_data);
 
+#ifndef GST_DISABLE_DEPRECATED
 #ifndef GST_DISABLE_LOADSAVE
 void        gst_class_signal_emit_by_name   (GstObject     * object,
                                              const gchar   * name,
@@ -328,7 +346,7 @@ void        gst_class_signal_emit_by_name   (GstObject     * object,
 #pragma GCC poison gst_class_signal_emit_by_name
 #endif
 #endif
-
+#endif
 
 G_END_DECLS
 
