@@ -35,6 +35,7 @@
 #include "gstregistry.h"
 #include "gstinfo.h"
 
+#include <stdio.h>
 #include <string.h>
 
 #define GST_CAT_DEFAULT GST_CAT_PLUGIN_LOADING
@@ -51,8 +52,7 @@ gst_plugin_feature_class_init (GstPluginFeatureClass * klass)
 {
   parent_class = g_type_class_peek_parent (klass);
 
-  G_OBJECT_CLASS (klass)->finalize =
-      GST_DEBUG_FUNCPTR (gst_plugin_feature_finalize);
+  G_OBJECT_CLASS (klass)->finalize = gst_plugin_feature_finalize;
 }
 
 static void
@@ -258,6 +258,63 @@ gst_plugin_feature_list_free (GList * list)
   g_list_free (list);
 }
 
+/**
+ * gst_plugin_feature_list_copy:
+ * @list: list of #GstPluginFeature
+ *
+ * Copies the list of features. Caller should call @gst_plugin_feature_list_free
+ * when done with the list.
+ *
+ * Returns: a copy of @list, with each feature's reference count incremented.
+ *
+ * Since: 0.10.26
+ */
+GList *
+gst_plugin_feature_list_copy (GList * list)
+{
+  GList *new_list = NULL;
+
+  if (G_LIKELY (list)) {
+    GList *last;
+
+    new_list = g_list_alloc ();
+    new_list->data = g_object_ref ((GObject *) list->data);
+    new_list->prev = NULL;
+    last = new_list;
+    list = list->next;
+    while (list) {
+      last->next = g_list_alloc ();
+      last->next->prev = last;
+      last = last->next;
+      last->data = g_object_ref ((GObject *) list->data);
+      list = list->next;
+    }
+    last->next = NULL;
+  }
+
+  return new_list;
+}
+
+/**
+ * gst_plugin_feature_list_debug:
+ * @list: a #GList of plugin features
+ *
+ * Debug the plugin feature names in @list.
+ *
+ * Since: 0.10.31
+ */
+void
+gst_plugin_feature_list_debug (GList * list)
+{
+#ifndef GST_DISABLE_GST_DEBUG
+  while (list) {
+    GST_DEBUG ("%s",
+        gst_plugin_feature_get_name ((GstPluginFeature *) list->data));
+    list = list->next;
+  }
+#endif
+}
+
 /**
  * gst_plugin_feature_check_version:
  * @feature: a feature
@@ -331,3 +388,36 @@ gst_plugin_feature_check_version (GstPluginFeature * feature,
 
   return ret;
 }
+
+/**
+ * gst_plugin_feature_rank_compare_func:
+ * @p1: a #GstPluginFeature
+ * @p2: a #GstPluginFeature
+ *
+ * Compares the two given #GstPluginFeature instances. This function can be
+ * used as a #GCompareFunc when sorting by rank and then by name.
+ *
+ * Returns: negative value if the rank of p1 > the rank of p2 or the ranks are
+ * equal but the name of p1 comes before the name of p2; zero if the rank
+ * and names are equal; positive value if the rank of p1 < the rank of p2 or the
+ * ranks are equal but the name of p2 comes after the name of p1
+ *
+ * Since: 0.10.31
+ */
+gint
+gst_plugin_feature_rank_compare_func (gconstpointer p1, gconstpointer p2)
+{
+  GstPluginFeature *f1, *f2;
+  gint diff;
+
+  f1 = (GstPluginFeature *) p1;
+  f2 = (GstPluginFeature *) p2;
+
+  diff = f2->rank - f1->rank;
+  if (diff != 0)
+    return diff;
+
+  diff = strcmp (f2->name, f1->name);
+
+  return diff;
+}
