@@ -29,13 +29,13 @@
 
 G_BEGIN_DECLS
 
-#define GST_TYPE_PLUGIN_FEATURE 		(gst_plugin_feature_get_type())
-#define GST_PLUGIN_FEATURE(obj) 		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_PLUGIN_FEATURE, GstPluginFeature))
-#define GST_IS_PLUGIN_FEATURE(obj) 		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_PLUGIN_FEATURE))
-#define GST_PLUGIN_FEATURE_CLASS(klass) 	(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_PLUGIN_FEATURE, GstPluginFeatureClass))
-#define GST_IS_PLUGIN_FEATURE_CLASS(klass) 	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_PLUGIN_FEATURE))
-#define GST_PLUGIN_FEATURE_GET_CLASS(obj) 	(G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_PLUGIN_FEATURE, GstPluginFeatureClass))
-#define GST_PLUGIN_FEATURE_CAST(obj) 		((GstPluginFeature*)(obj))
+#define GST_TYPE_PLUGIN_FEATURE                 (gst_plugin_feature_get_type())
+#define GST_PLUGIN_FEATURE(obj)                 (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_PLUGIN_FEATURE, GstPluginFeature))
+#define GST_IS_PLUGIN_FEATURE(obj)              (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_PLUGIN_FEATURE))
+#define GST_PLUGIN_FEATURE_CLASS(klass)         (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_PLUGIN_FEATURE, GstPluginFeatureClass))
+#define GST_IS_PLUGIN_FEATURE_CLASS(klass)      (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_PLUGIN_FEATURE))
+#define GST_PLUGIN_FEATURE_GET_CLASS(obj)       (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_PLUGIN_FEATURE, GstPluginFeatureClass))
+#define GST_PLUGIN_FEATURE_CAST(obj)            ((GstPluginFeature*)(obj))
 
 /**
  * GST_PLUGIN_FEATURE_NAME:
@@ -56,12 +56,12 @@ typedef struct _GstPluginFeatureClass GstPluginFeatureClass;
  * @GST_RANK_PRIMARY: will be chosen first
  *
  * Element priority ranks. Defines the order in which the autoplugger (or
- * similar rank-picking mechanisms) will choose this element over an alternative
- * one with the same function.
+ * similar rank-picking mechanisms, such as e.g. gst_element_make_from_uri())
+ * will choose this element over an alternative one with the same function.
  *
- * The rank is a unsigned integer ranging from 0 (GST_RANK_NONE) to 256
- * (GST_RANK_PRIMARY). These constants serve as a rough guidiance for defining
- * the rank of a #GstPlugin using gst_plugin_feature_set_rank().
+ * These constants serve as a rough guidance for defining the rank of a
+ * #GstPluginFeature. Any value is valid, including values bigger than
+ * @GST_RANK_PRIMARY.
  */
 typedef enum {
   GST_RANK_NONE                 = 0,
@@ -76,21 +76,21 @@ typedef enum {
  * Opaque #GstPluginFeature structure.
  */
 struct _GstPluginFeature {
-  GstObject 	 object;
+  GstObject      object;
 
   /*< private >*/
   gboolean       loaded;
-  gchar 	*name;
-  guint   	 rank;
+  gchar         *name;
+  guint          rank;
 
-  gchar *plugin_name;
+  const gchar   *plugin_name;
 
   /*< private >*/
   gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstPluginFeatureClass {
-  GstObjectClass	parent_class;
+  GstObjectClass        parent_class;
 
   /*< private >*/
   gpointer _gst_reserved[GST_PADDING];
@@ -104,8 +104,8 @@ struct _GstPluginFeatureClass {
  * Structure used for filtering based on @name and @type.
  */
 typedef struct {
-  const gchar 	*name;
-  GType		 type;
+  const gchar   *name;
+  GType          type;
 } GstTypeNameData;
 
 /**
@@ -123,25 +123,35 @@ typedef gboolean        (*GstPluginFeatureFilter)       (GstPluginFeature *featu
                                                          gpointer user_data);
 
 /* normal GObject stuff */
-GType		gst_plugin_feature_get_type		(void);
+GType           gst_plugin_feature_get_type             (void);
 
 GstPluginFeature *
-                gst_plugin_feature_load         	(GstPluginFeature *feature);
+                gst_plugin_feature_load                 (GstPluginFeature *feature);
 
-gboolean	gst_plugin_feature_type_name_filter	(GstPluginFeature *feature,
-							 GstTypeNameData *data);
+gboolean        gst_plugin_feature_type_name_filter     (GstPluginFeature *feature,
+                                                         GstTypeNameData *data);
 
-void		gst_plugin_feature_set_rank		(GstPluginFeature *feature, guint rank);
-void		gst_plugin_feature_set_name		(GstPluginFeature *feature, const gchar *name);
-guint		gst_plugin_feature_get_rank		(GstPluginFeature *feature);
-G_CONST_RETURN gchar *gst_plugin_feature_get_name	(GstPluginFeature *feature);
+void            gst_plugin_feature_set_rank             (GstPluginFeature *feature, guint rank);
+void            gst_plugin_feature_set_name             (GstPluginFeature *feature, const gchar *name);
+guint           gst_plugin_feature_get_rank             (GstPluginFeature *feature);
+G_CONST_RETURN gchar *gst_plugin_feature_get_name       (GstPluginFeature *feature);
 
 void            gst_plugin_feature_list_free            (GList *list);
+GList          *gst_plugin_feature_list_copy            (GList *list);
+void            gst_plugin_feature_list_debug           (GList *list);
+
+#ifndef GST_DISABLE_GST_DEBUG
+#define GST_PLUGIN_FEATURE_LIST_DEBUG(list) gst_plugin_feature_list_debug(list)
+#else
+#define GST_PLUGIN_FEATURE_LIST_DEBUG(list)
+#endif
 
 gboolean        gst_plugin_feature_check_version        (GstPluginFeature *feature,
                                                          guint             min_major,
                                                          guint             min_minor,
                                                          guint             min_micro);
+gint            gst_plugin_feature_rank_compare_func    (gconstpointer p1,
+							 gconstpointer p2);
 
 G_END_DECLS
 
