@@ -23,6 +23,7 @@
 #include <gst/gstconfig.h>
 #include <glib-object.h>
 #include <gst/gstclock.h>
+#include <gst/gstdatetime.h>
 #include <gst/glib-compat.h>
 
 G_BEGIN_DECLS
@@ -95,6 +96,9 @@ GstStructure *          gst_structure_new                  (const gchar *
 GstStructure *          gst_structure_new_valist           (const gchar *            name,
 						            const gchar *            firstfield,
 							    va_list                  varargs);
+GstStructure *          gst_structure_id_new               (GQuark                   name_quark,
+                                                            GQuark                   field_quark,
+                                                            ...);
 GstStructure *          gst_structure_copy                 (const GstStructure      *structure);
 void			gst_structure_set_parent_refcount  (GstStructure            *structure,
                                                             gint            *refcount);
@@ -113,12 +117,44 @@ void                    gst_structure_id_set_value         (GstStructure
 void                    gst_structure_set_value            (GstStructure            *structure,
 							    const gchar             *fieldname,
 							    const GValue            *value);
+void                    gst_structure_id_take_value         (GstStructure            *structure,
+							    GQuark                   field,
+							    GValue                  *value);
+void                    gst_structure_take_value            (GstStructure            *structure,
+							    const gchar             *fieldname,
+							    GValue                  *value);
 void                    gst_structure_set                  (GstStructure            *structure,
 							    const gchar             *fieldname,
 							    ...) G_GNUC_NULL_TERMINATED;
+
 void                    gst_structure_set_valist           (GstStructure            *structure,
 							    const gchar             *fieldname,
 							    va_list varargs);
+
+void                    gst_structure_id_set                (GstStructure            *structure,
+							    GQuark                   fieldname,
+							    ...) G_GNUC_NULL_TERMINATED;
+
+void                    gst_structure_id_set_valist         (GstStructure            *structure,
+							    GQuark                   fieldname,
+							    va_list varargs);
+
+gboolean                gst_structure_get_valist           (const GstStructure      *structure,
+                                                            const char              *first_fieldname,
+                                                            va_list                  args);
+
+gboolean                gst_structure_get                  (const GstStructure      *structure,
+                                                            const char              *first_fieldname,
+                                                            ...) G_GNUC_NULL_TERMINATED;
+
+gboolean                gst_structure_id_get_valist        (const GstStructure      *structure,
+                                                            GQuark                   first_field_id,
+                                                            va_list                  args);
+
+gboolean                gst_structure_id_get               (const GstStructure      *structure,
+                                                            GQuark                   first_field_id,
+                                                            ...) G_GNUC_NULL_TERMINATED;
+
 G_CONST_RETURN GValue * gst_structure_id_get_value         (const GstStructure      *structure,
 							    GQuark                   field);
 G_CONST_RETURN GValue * gst_structure_get_value            (const GstStructure      *structure,
@@ -143,6 +179,11 @@ gboolean                gst_structure_map_in_place         (GstStructure
 							    gpointer                 user_data);
 gint                    gst_structure_n_fields             (const GstStructure      *structure);
 const gchar *           gst_structure_nth_field_name       (const GstStructure      *structure, guint index);
+gboolean                gst_structure_id_has_field         (const GstStructure      *structure,
+							    GQuark                   field);
+gboolean                gst_structure_id_has_field_typed   (const GstStructure      *structure,
+							    GQuark                   field,
+							    GType                    type);
 gboolean                gst_structure_has_field            (const GstStructure      *structure,
 							    const gchar             *fieldname);
 gboolean                gst_structure_has_field_typed      (const GstStructure      *structure,
@@ -156,6 +197,9 @@ gboolean                gst_structure_get_boolean          (const GstStructure
 gboolean                gst_structure_get_int              (const GstStructure      *structure,
 							    const gchar             *fieldname,
 							    gint                    *value);
+gboolean                gst_structure_get_uint             (const GstStructure      *structure,
+							    const gchar             *fieldname,
+							    guint                   *value);
 gboolean                gst_structure_get_fourcc           (const GstStructure      *structure,
 							    const gchar             *fieldname,
 							    guint32                 *value);
@@ -165,6 +209,9 @@ gboolean                gst_structure_get_double           (const GstStructure
 gboolean                gst_structure_get_date             (const GstStructure      *structure,
 							    const gchar             *fieldname,
                                                             GDate                  **value);
+gboolean                gst_structure_get_date_time        (const GstStructure      *structure,
+							    const gchar             *fieldname,
+                                                            GstDateTime              **value);
 gboolean                gst_structure_get_clock_time       (const GstStructure      *structure,
 							    const gchar             *fieldname,
 							    GstClockTime            *value);
@@ -193,6 +240,9 @@ gboolean                 gst_structure_fixate_field_nearest_double (GstStructure
 gboolean                 gst_structure_fixate_field_boolean (GstStructure *structure,
 									 const char   *field_name,
 									 gboolean        target);
+gboolean                 gst_structure_fixate_field_string (GstStructure *structure,
+									 const char   *field_name,
+									 const gchar  *target);
 gboolean                 gst_structure_fixate_field_nearest_fraction (GstStructure *structure,
 									 const char   *field_name,
 									 const gint target_numerator,
