@@ -30,8 +30,12 @@
  * Additionally one can load saved pipelines into the gst-editor to inspect the
  * graph.
  *
- * #GstElement implementations need to override gst_object_save_thyself() and
- * gst_object_restore_thyself().
+ * #GstElement implementations need to override the #GstObjectClass.save_thyself()
+ * and #GstObjectClass.restore_thyself() virtual functions of #GstObject.
+ *
+ * Deprecated: This feature is deprecated pipeline serialization to XML is
+ * broken for all but the most simple pipelines. It will most likely be
+ * removed in future. Don't use it.
  */
 
 #include "gst_private.h"
@@ -41,14 +45,71 @@
 #include "gstinfo.h"
 #include "gstbin.h"
 
+#ifdef GST_DISABLE_DEPRECATED
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
+#include <libxml/parser.h>
+xmlNodePtr gst_object_save_thyself (const GstObject * object,
+    xmlNodePtr parent);
+GstObject *gst_object_load_thyself (xmlNodePtr parent);
+void gst_object_restore_thyself (GstObject * object, GstXmlNodePtr self);
+
+#define GST_TYPE_XML 		(gst_xml_get_type ())
+#define GST_XML(obj) 		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_XML, GstXML))
+#define GST_IS_XML(obj) 	(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_XML))
+#define GST_XML_CLASS(klass) 	(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_XML, GstXMLClass))
+#define GST_IS_XML_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_XML))
+#define GST_XML_GET_CLASS(obj) 	(G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_XML, GstXMLClass))
+
+typedef struct _GstXML GstXML;
+typedef struct _GstXMLClass GstXMLClass;
+
+struct _GstXML
+{
+  GstObject object;
+
+  /*< public > */
+  GList *topelements;
+
+  xmlNsPtr ns;
+
+  /*< private > */
+  gpointer _gst_reserved[GST_PADDING];
+};
+
+struct _GstXMLClass
+{
+  GstObjectClass parent_class;
+
+  /* signal callbacks */
+  void (*object_loaded) (GstXML * xml, GstObject * object, xmlNodePtr self);
+  void (*object_saved) (GstXML * xml, GstObject * object, xmlNodePtr self);
+
+  gpointer _gst_reserved[GST_PADDING];
+};
+
+GType gst_xml_get_type (void);
+xmlDocPtr gst_xml_write (GstElement * element);
+gint gst_xml_write_file (GstElement * element, FILE * out);
+GstXML *gst_xml_new (void);
+gboolean gst_xml_parse_doc (GstXML * xml, xmlDocPtr doc, const guchar * root);
+gboolean gst_xml_parse_file (GstXML * xml, const guchar * fname,
+    const guchar * root);
+gboolean gst_xml_parse_memory (GstXML * xml, guchar * buffer, guint size,
+    const gchar * root);
+GstElement *gst_xml_get_element (GstXML * xml, const guchar * name);
+GList *gst_xml_get_topelements (GstXML * xml);
+GstElement *gst_xml_make_element (xmlNodePtr cur, GstObject * parent);
+#endif
+#endif
+
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
+
 enum
 {
   OBJECT_LOADED,
   LAST_SIGNAL
 };
 
-static void gst_xml_class_init (GstXMLClass * klass);
-static void gst_xml_init (GstXML * xml);
 static void gst_xml_dispose (GObject * object);
 
 static void gst_xml_object_loaded (GstObject * private, GstObject * object,
@@ -57,40 +118,16 @@ static void gst_xml_object_loaded (GstObject * private, GstObject * object,
 static GstObjectClass *parent_class = NULL;
 static guint gst_xml_signals[LAST_SIGNAL] = { 0 };
 
-GType
-gst_xml_get_type (void)
-{
-  static GType xml_type = 0;
-
-  if (G_UNLIKELY (xml_type == 0)) {
-    static const GTypeInfo xml_info = {
-      sizeof (GstXMLClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gst_xml_class_init,
-      NULL,
-      NULL,
-      sizeof (GstXML),
-      0,
-      (GInstanceInitFunc) gst_xml_init,
-      NULL
-    };
-
-    xml_type = g_type_register_static (GST_TYPE_OBJECT, "GstXML", &xml_info, 0);
-  }
-  return xml_type;
-}
+G_DEFINE_TYPE (GstXML, gst_xml, GST_TYPE_OBJECT);
 
 static void
 gst_xml_class_init (GstXMLClass * klass)
 {
-  GObjectClass *gobject_class;
-
-  gobject_class = (GObjectClass *) klass;
+  GObjectClass *gobject_class = (GObjectClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_xml_dispose);
+  gobject_class->dispose = gst_xml_dispose;
 
   /* FIXME G_TYPE_POINTER should be GType of xmlNodePtr
    * (ensonic) can't be fixed, as libxml does not use GObject (unfortunately)
@@ -139,7 +176,7 @@ gst_xml_dispose (GObject * object)
 GstXML *
 gst_xml_new (void)
 {
-  return GST_XML (g_object_new (GST_TYPE_XML, NULL));
+  return GST_XML (g_object_newv (GST_TYPE_XML, 0, NULL));
 }
 
 /**
@@ -369,8 +406,7 @@ gst_xml_object_loaded (GstObject * private, GstObject * object, xmlNodePtr self,
   GstXML *xml = GST_XML (data);
 
   /* FIXME check that this element was created from the same xmlDocPtr... */
-  g_signal_emit (G_OBJECT (xml), gst_xml_signals[OBJECT_LOADED], 0, object,
-      self);
+  g_signal_emit (xml, gst_xml_signals[OBJECT_LOADED], 0, object, self);
 }
 
 /**
@@ -488,3 +524,18 @@ gst_xml_make_element (xmlNodePtr cur, GstObject * parent)
 
   return element;
 }
+
+#else
+
+/* FIXME: keep this dummy _get_type function around for now, so
+ * gobject-introspection doesn't fail in the GST_REMOVE_DEPRECATED and
+ * GST_DISABLE_LOADSAVE case */
+GType gst_xml_get_type (void);
+
+GType
+gst_xml_get_type (void)
+{
+  return g_pointer_type_register_static ("GstXML");
+}
+
+#endif
