@@ -85,6 +85,11 @@ struct _GstQueue {
   GstSegment sink_segment;
   GstSegment src_segment;
 
+  /* position of src/sink */
+  GstClockTime sinktime, srctime;
+  /* TRUE if either position needs to be recalculated */
+  gboolean sink_tainted, src_tainted;
+
   /* flowreturn when srcpad is paused */
   GstFlowReturn srcresult;
   gboolean      unexpected;
@@ -103,10 +108,18 @@ struct _GstQueue {
   gint leaky;
 
   GMutex *qlock;        /* lock for queue (vs object lock) */
+  gboolean waiting_add;
   GCond *item_add;      /* signals buffers now available for reading */
+  gboolean waiting_del;
   GCond *item_del;      /* signals space now available for writing */
 
   gboolean head_needs_discont, tail_needs_discont;
+  gboolean push_newsegment;
+  
+  gboolean silent;      /* don't emit signals */ 
+
+  /* whether the first new segment has been applied to src */
+  gboolean newseg_applied_to_src;
 };
 
 struct _GstQueueClass {
